/**
 * posix_File.
 *
 * @version     1.0 20/6/98
 * @author      Jan Newmarch.
 * Copyright &copy; Jan Newmarch.
 * Covered by GNU Library General Public License
 */

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <dlfcn.h>
#include <stdio.h>
#include <string.h>
#include "posix_File.h"
#include "Fields.h"
#include "PosixException.h"

/* #define DEBUG */

/*
 * Class:     posix_File
 * Method:    initializeConstants
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_posix_File_initializeConstants
  (JNIEnv *env, jclass cls)
{
  jfieldID fid;

  /* access permissions */
#ifdef DEBUG
  fprintf(stderr, "Setting File constants\n");
#endif
  setStaticInt(env, cls, "r_ok", R_OK);
  setStaticInt(env, cls, "w_ok", W_OK);
  setStaticInt(env, cls, "x_ok", X_OK);
  setStaticInt(env, cls, "f_ok", F_OK);
}

/*
 * Class:     posix_File
 * Method:    link
 * Signature: (Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_posix_File_link(JNIEnv *env, 
					   jobject obj, 
					   jstring jpath)
{
  int status;
  
  /* get C values from object */
  stringFieldStruct s = getStringField(env, obj, "path");
  char *oldPath = s.str;
  jstring jOldPath = s.jstr;

  /* get C values from Java params */
  const char *newPath = (*env) -> GetStringUTFChars(env, jpath, 0);

# ifdef DEBUG
    fprintf(stderr, "attempting link of %s to %s\n", 
	    oldPath, newPath);
# endif

  /* process */
  status = link(oldPath, newPath);  
  
  /* cleanup */
  (*env) -> ReleaseStringUTFChars(env, jOldPath, oldPath);
  (*env) -> ReleaseStringUTFChars(env, jpath, newPath);

  /* throw exception? */
  if (status != 0) {
    throwPosixException(env, errno, strerror(errno));
  }
}

/*
 * Class:     posix_File
 * Method:    unlink
 * Signature: ()V
 */

JNIEXPORT void JNICALL Java_posix_File_unlink(JNIEnv *env, 
					     jobject obj) 
{
  int status;

  /* get C values from object */
  stringFieldStruct s = getStringField(env, obj, "path");
  char *path = s.str;
  jstring jpath = s.jstr;

# ifdef DEBUG
    fprintf(stderr, "attempting unlink of %s\n", path);
# endif

  /* process */
  status = unlink(path);  

  /* cleanup */
  (*env) -> ReleaseStringUTFChars(env, jpath, path);

  /* set error? */
  if (status != 0) {
    throwPosixException(env, errno, strerror(errno));
  }
}


/*
 * Class:     posix_File
 * Method:    rename
 * Signature: (Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_posix_File_rename
  (JNIEnv *env, jobject obj, jstring newName)
{
  int status;

  /* get C values from object */
  stringFieldStruct s = getStringField(env, obj, "path");
  char *oldPath = s.str;
  jstring jOldPath = s.jstr;

  /* get C values from Java params */
  const char *newPath = (*env) -> GetStringUTFChars(env, newName, 0);

# ifdef DEBUG
  fprintf(stderr, "attempting rename of %s to %s\n", 
	  oldPath, newPath);
# endif

  /* process */
  status = rename(oldPath, newPath);  
  if (status == 0) {
    /* now we have to change the Java path to reflect this change */
    setStringField(env, obj, "path", newPath);
  }

  /* cleanup */
  (*env) -> ReleaseStringUTFChars(env, jOldPath, oldPath);
  (*env) -> ReleaseStringUTFChars(env, newName, newPath);

  /* set error? */
  if (status != 0) {
    throwPosixException(env, errno, strerror(errno));
  }
}

/*
 * Class:     posix_File
 * Method:    access
 * Signature: (I)V
 */

JNIEXPORT void JNICALL Java_posix_File_access
  (JNIEnv *env, jobject obj, jint mode)
{
  int status;

  /* get C values from object */
  stringFieldStruct s = getStringField(env, obj, "path");
  char *path = s.str;
  jstring jpath = s.jstr;

  /* get C values from Java params */

# ifdef DEBUG
    fprintf(stderr, "attempting access of %s\n", path);
# endif

  /* process */
  status = access(path, mode);  
  
  /* cleanup */
  (*env) -> ReleaseStringUTFChars(env, jpath, path);

  /* set error? */
  if (status != 0) {
    throwPosixException(env, errno, strerror(errno));
  }
}

/*
 * Class:     posix_File
 * Method:    chmod
 * Signature: (I)V
 */

JNIEXPORT void JNICALL Java_posix_File_chmod
  (JNIEnv *env, jobject obj, jint jmode)
{
  int status;
  mode_t mode = (mode_t) jmode;

  /* get C values from object */
  stringFieldStruct s = getStringField(env, obj, "path");
  char *path = s.str;
  jstring jpath = s.jstr;

# ifdef DEBUG
    fprintf(stderr, "attempting chmod of %s\n", path);
# endif

  /* process */
  status = chmod(path, mode);  
  
  /* cleanup */
  (*env) -> ReleaseStringUTFChars(env, jpath, path);

  /* set error? */
  if (status != 0) {
    throwPosixException(env, errno, strerror(errno));
  }
}

/*
 * Class:     posix_File
 * Method:    chown
 * Signature: (II)V
 */

JNIEXPORT void JNICALL Java_posix_File_chown
   (JNIEnv *env , jobject obj, 
    jint jowner, jint jgroup)
{
  int status;

  uid_t owner = jowner;
  gid_t group = jgroup;

  /* get C values from object */
  stringFieldStruct s = getStringField(env, obj, "path");
  char *path = s.str;
  jstring jpath = s.jstr;

# ifdef DEBUG
    fprintf(stderr, "attempting chown of %s\n", path);
# endif

  /* process */
  status = chown(path, owner, group);  
  
  /* cleanup */
  (*env) -> ReleaseStringUTFChars(env, jpath, path);

  /* set error? */
  if (status != 0) {
    throwPosixException(env, errno, strerror(errno));
  }
}


