/**
 * posix_OpenFile.
 *
 * @version     1.0 20/6/98
 * @author      Jan Newmarch.
 * Copyright &copy; Jan Newmarch.
 * Covered by GNU Library General Public License
 */


#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>
#include "posix_OpenFile.h"
#include "PosixException.h"
#include "Fields.h"

/* #define DEBUG */


/*
 * Class:     posix_OpenFile
 * Method:    initializeConstants
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_posix_OpenFile_initializeConstants
  (JNIEnv *env, jclass cls)
{
    setStaticInt(env, cls, "o_rdonly", O_RDONLY);
    setStaticInt(env, cls, "o_wronly", O_WRONLY);
    setStaticInt(env, cls, "o_rdwr", O_RDWR);
    setStaticInt(env, cls, "o_append", O_APPEND);
    setStaticInt(env, cls, "o_creat", O_CREAT);
    setStaticInt(env, cls, "o_excl", O_EXCL);
    setStaticInt(env, cls, "o_noctty", O_NOCTTY);
    setStaticInt(env, cls, "o_nonblock", O_NONBLOCK);
    setStaticInt(env, cls, "o_trunc", O_TRUNC);
}

/*
 * Class:     posix_OpenFile
 * Method:    open
 * Signature: (Ljava/lang/String;I)I
 */
JNIEXPORT void JNICALL Java_posix_OpenFile_open__Ljava_lang_String_2I
                                          (JNIEnv *env, 
                                           jobject obj, 
                                           jstring jpath, 
                                           jint oflag)
{
  int status;

  /* get C values from Java params */
  const char *path = (*env) -> GetStringUTFChars(env, jpath, 0);

  /* get file descriptor Java id */
  jclass class = (*env) -> GetObjectClass(env, obj);
  jfieldID jfid = (*env) -> GetFieldID(env, class, "fd", "I");

# ifdef DEBUG
    fprintf(stderr, "attempting to open %s for access\n", path);
# endif

  /* process */
  status = open(path, oflag);

# ifdef DEBUG
    if (status == -1)
      perror("");
    fprintf(stderr, "opened %s on file descriptor %d with flag %d\n", 
path, status, oflag);
# endif

  /* set Java file descriptor value */
  (*env) -> SetIntField(env, obj, jfid, status);
  
  /* cleanup */
  (*env) -> ReleaseStringUTFChars(env, jpath, path);

  /* set error? */
  if (status < 0) {
    /* set error */
    throwPosixException(env, errno, strerror(errno));
  }
}

/*
 * Class:     posix_OpenFile
 * Method:    open
 * Signature: (Ljava/lang/String;II)V
 */
JNIEXPORT void JNICALL Java_posix_OpenFile_open__Ljava_lang_String_2II
  (JNIEnv *env, jobject obj, jstring jpath, 
   jint oflag, jint mode)
{
  int status;

  /* get C values from Java params */
  const char *path = (*env) -> GetStringUTFChars(env, jpath, 0);

  /* get file descriptor Java id */
  jclass class = (*env) -> GetObjectClass(env, obj);
  jfieldID jfid = (*env) -> GetFieldID(env, class, "fd", "I");

# ifdef DEBUG
    fprintf(stderr, "attempting to open %s for access\n", path);
# endif

  /* process */
  status = open(path, oflag, mode);

# ifdef DEBUG
    if (status == -1)
      perror("");
    fprintf(stderr, "opened %s on file descriptor %d with flag %d\n", 
path, status, oflag);
# endif

  /* set Java file descriptor value */
  (*env) -> SetIntField(env, obj, jfid, status);
  
  /* cleanup */
  (*env) -> ReleaseStringUTFChars(env, jpath, path);

  /* set error? */
  if (status < 0) {
    /* set error */
    throwPosixException(env, errno, strerror(errno));
  }
}

/*
 * Class:     posix_OpenFile
 * Method:    creat
 * Signature: (Ljava/lang/String;I)V
 */
JNIEXPORT void JNICALL Java_posix_OpenFile_creat
  (JNIEnv *env, jobject obj, jstring jpath, jint mode)
{
  int status;

  /* get C values from Java params */
  const char *path = (*env) -> GetStringUTFChars(env, jpath, 0);

  /* get file descriptor Java id */
  jclass class = (*env) -> GetObjectClass(env, obj);
  jfieldID jfid = (*env) -> GetFieldID(env, class, "fd", "I");

# ifdef DEBUG
    fprintf(stderr, "attempting to open %s for access\n", path);
# endif

  /* process */
  status = creat(path, mode);

# ifdef DEBUG
    if (status == -1)
      perror("");
    fprintf(stderr, "created %s on file descriptor %d with mode %d\n", 
path, status, mode);
# endif

  /* set Java file descriptor value */
  (*env) -> SetIntField(env, obj, jfid, status);
  
  /* cleanup */
  (*env) -> ReleaseStringUTFChars(env, jpath, path);

  /* set error? */
  if (status < 0) {
    /* set error */
    throwPosixException(env, errno, strerror(errno));
  }
}

/*
 * Class:     posix_OpenFile
 * Method:    close
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_posix_OpenFile_close
  (JNIEnv *env , jobject obj)
{
  int status;

  /* get file descriptor Java id */
  jclass class = (*env) -> GetObjectClass(env, obj);
  jfieldID jfid = (*env) -> GetFieldID(env, class, "fd", "I");

  /* get file descriptor */
  int fd = (int) (*env) -> GetIntField(env, obj, jfid);

  /* process */
  status = close(fd);

  /* set error? */
  if (status != 0) {
    /* set error */
    throwPosixException(env, errno, strerror(errno));
  }
}

/*
 * Class:     posix_OpenFile
 * Method:    dup2
 * Signature: (I)I
 */
JNIEXPORT void JNICALL Java_posix_OpenFile_dup2
  (JNIEnv *env, jobject obj, jint fd2)
{
  int status;

  /* THIS FUNCTION DOES NOT CHECK READ/WRITE OF OBJECT */
  /* get C values from Java params */


  /* get file descriptor Java id */
  jclass class = (*env) -> GetObjectClass(env, obj);
  jfieldID jfid = (*env) -> GetFieldID(env, class, "fd", "I");

  /* get file descriptor */
  int fd = (int) (*env) -> GetIntField(env, obj, jfid);

# ifdef DEBUG
    fprintf(stderr, "attempting to dup2 %d to %d\n", fd, fd2);
# endif

  /* process */
  status = dup2(fd, fd2);

# ifdef DEBUG
    if (status == -1)
      perror("OpenFile.dup2 - ");
    else
      fprintf(stderr, "dup2 ok\n");
# endif

  /* set Java file descriptor value */
  // (*env) -> SetIntField(env, obj, jfid, status);
  
  /* cleanup */


  /* set error? */
  if (status < 0) {
    /* set error */
    throwPosixException(env, errno, strerror(errno));
  }
}
