/**
 * Posix class Stat.
 *
 * @version     1.0 20/6/98
 * @author      Jan Newmarch.
 * Copyright &copy; Jan Newmarch.
 * Covered by GNU Library General Public License
 */



package posix;

public class Stat {

    /**
     * read, write, execute for user
     */
    public final static int RWXU;

    /**
     * read for user
     */
    public final static int RUSR;

    /**
     * write for user
     */
    public final static int WUSR;

    /**
     * execute for user
     */
    public final static int XUSR;

    /**
     * read, write, execute for group
     */
    public final static int RWXG;

    /**
     * read for group
     */
    public final static int RGRP;

    /**
     * write for group
     */
    public final static int WGRP;

    /**
     * execute for group
     */
    public final static int XGRP;


    /**
     * read, write, execute for other
     */
    public final static int RWXO;

    /**
     * read for other
     */
    public final static int ROTH;

    /**
     * write for other
     */
    public final static int WOTH;

    /**
     * execute for other
     */
    public final static int XOTH;

    /**
     * set user id on execution
     */
    public final static int SUID;

    /**
     * set group id on execution
     */
    public final static int SGID;

    private static int s_irwxu;
    private static int s_irusr;
    private static int s_iwusr;
    private static int s_ixusr;

    private static int s_irwxg;
    private static int s_irgrp;
    private static int s_iwgrp;
    private static int s_ixgrp;
    private static int s_irwxo;

    private static int s_iroth;
    private static int s_iwoth;
    private static int s_ixoth;

    private static int s_isuid;
    private static int s_isgid;

    private int st_mode;
    private int st_ino;
    private int st_dev;
    private int st_nlink;
    private int st_uid;
    private int st_gid;
    private int st_size;
    private int st_atime;
    private int st_mtime;
    private int st_ctime;

    static {
	initializeConstants(); // sets s_irusr, etc

	RWXU = s_irwxu;
	RUSR = s_irusr;
	WUSR = s_iwusr;
	XUSR = s_ixusr;

	RWXG = s_irwxg;
	RGRP = s_irgrp;
	WGRP = s_iwgrp;
	XGRP = s_ixgrp;

	RWXO = s_irwxo;
	ROTH = s_iroth;
	WOTH = s_iwoth;
	XOTH = s_ixoth;

	SUID = s_isuid;
	SGID = s_isgid;
    }

    protected static native void initializeConstants();

    /**
     * Constructs a Stat from the filename.
     * @exception PosixException EACCES, ENAMETOOLONG, ENOENT,
     * ENOTDIR.
     */
    public Stat(String path) throws PosixException {
	if (stat(path) == -1) {
	    throw new PosixException();
	}
    }

    /**
     * Constructs a Stat from the File object.
     * @exception PosixException EACCES, ENAMETOOLONG, ENOENT,
     * ENOTDIR.
     */
    public Stat(posix.File file) throws PosixException {
	if (stat(file.getPath()) == -1) {
	    throw new PosixException();
	}
    }

    /**
     * Constructs a Stat from an OpenFile.
     * @exception PosixException EACCES, ENAMETOOLONG, ENOENT,
     * ENOTDIR, EBADF.
     */
    public Stat(posix.OpenFile fd) throws PosixException {
	if (stat(fd.getPath()) == -1) {
	    throw new PosixException();
	}
    }

    protected native int stat(String path);

    /**
     * True if the file is a directory file
     */
    public native boolean isDirectory();

    /**
     * True if the file is a character special file
     */
    public native boolean isCharacterSpecial();

    /**
     * True if the file is a block special file
     */
    public native boolean isBlockSpecial();

    /**
     * True if the file is a pipe or FIFO special file
     */
    public native boolean isRegularFile();

    /**
     * True if the file is a directory file
     */
    public native boolean isFIFO();

    /**
     * Return the permissions mode of the file.
     * Posix type: mode_t, typically u_short
     * @see posix.File
     */
    public int getMode() {
	return st_mode;
    }

    /**
     * Returns the file serial number.
     * Posix type: ino_t, typically unsigned long
     */
    public long getInode() {
	return st_ino;
    }

    /**
     * Returns the ID of device within this file.
     * Posix type: dev_t, typically unsigned short
     */
    public int getDevice() {
	return st_dev;
    }

    /**
     * Returns the number of links to this file.
     * Posix type: nlink_t, typically unsigned short
     */
    public int getLinkCount() {
	return st_nlink;
    }

    /**
     * Returns user id of the file's owner.
     * Posix type: uid_t, typically unsigned int
     */
    public int getUID() {
	return st_uid;
    }

    /**
     * Returns group id of the file's owner.
     * Posix type: gid_t, typically unsigned int
     */
    public int getGID() {
	return st_gid;
    }

    /**
     * Returns size of the file for regular types.
     * For other file types, the size of the file is
     * unspecified.
     * Posix type: off_t, typically long
     */
    public long getSize() {
	return st_size;
    }

    /**
     * Returns the time of last access.
     * Posix type: time_t, typically long
     */
    public long getAccessTime() {
	return st_atime;
    }

    /**
     * Returns the time of last data modification.
     * Posix type: time_t, typically long
     */
    public long getModifiedTime() {
	return st_mtime;
    }

    /**
     * Returns the time of last status change.
     * Posix type: time_t, typically long
     */
    public long getStatusChange() {
	return st_ctime;
    }
}
	
